@@ -5,6 +5,15 @@
  *
  * Revisions:
  *
+ *     11/03/00  [rcg]
+ *              - Ignore SIGCHLD on non-BSD systems.
+ *              - HUP signal now closes and reopens log.
+ *              - Log to specified facility.
+ *              - Corrected '<strings.h>' to '<string.h>'
+ *
+ *     10/10/00  [rcg]
+ *              - Delete duplicate '#include <sys/wait.h>'
+ *
  *     09/29/00  [rcg]
  *              - Revert signals to default in child (avoids conflict
  *                with BSDI authentication).
@@ -38,9 +47,8 @@
 #include <arpa/inet.h>
 #include <stdarg.h>
 #include <signal.h>
-#include <strings.h>
+#include <string.h>
 #include <ctype.h>
-#include <sys/wait.h>
 #include <limits.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -109,6 +117,7 @@ void    err_dump ( WHENCE, const char *format, ... );
 void    my_perror   ( void );
 char   *sys_err_str ( void );
 int     reaper  ( SIGPARAM );
+int     hupit   ( SIGPARAM );
 int     cleanup ( SIGPARAM );
 
 
@@ -122,6 +131,7 @@ int             Qargc       = 0;
 BOOL            Qargv_alloc = FALSE;
 BOOL            debug       = FALSE;
 FILE           *trace_file  = NULL;
+char           *trace_name  = NULL;
 char            msg_buf [ 2048 ] = "";
 FILE           *msg_out     = NULL;
 FILE           *err_out     = NULL;
@@ -243,7 +253,7 @@ char *argv[];
 #ifdef SYSLOG42
     openlog ( pname, 0 );
 #else
-    openlog ( pname, POP_LOGOPTS, LOG_DAEMON /*POP_FACILITY*/ );
+    openlog ( pname, POP_LOGOPTS, POP_FACILITY );
 #endif
 
     /*
@@ -260,6 +270,7 @@ char *argv[];
 
             case 't':
                 debug = TRUE;
+                trace_name = optarg;
                 trace_file = fopen ( optarg, "a" );
                 if ( trace_file == NULL )
                     err_dump ( HERE, "Unable to open trace file \"%s\"", optarg );
@@ -461,7 +472,13 @@ char *argv[];
 
     listen ( sockfd, 5 );
 
+#ifdef    BSD
     signal ( SIGCHLD, VOIDSTAR reaper  );
+#else /* not BSD */
+    signal ( SIGCHLD, SIG_IGN          ); /* ignore them and they'll go away */
+#endif /* BSD */
+
+    signal ( SIGHUP,  VOIDSTAR hupit   );
     signal ( SIGTERM, VOIDSTAR cleanup );
 
     while ( TRUE ) 
@@ -505,6 +522,7 @@ char *argv[];
              */
             signal ( SIGCHLD, SIG_DFL );
             signal ( SIGTERM, SIG_DFL );
+            signal ( SIGHUP,  SIG_DFL );
 
             close ( sockfd );
 #endif /* not _DEBUG */
@@ -695,9 +713,18 @@ reaper ( SIGPARAM )
     int   stts;
     pid_t child_pid = 0;
 
+
     TRACE  ( trace_file, POP_DEBUG, HERE, "reaper signal handler called" );
-    child_pid = wait3 ( &stts, WNOHANG, (struct rusage *) 0 );
-    TRACE  ( trace_file, POP_DEBUG, HERE, "wait3() returned %d", child_pid );
+
+    do
+    {
+        child_pid = wait3 ( &stts, WNOHANG, (struct rusage *) 0 );
+        TRACE  ( trace_file, POP_DEBUG, HERE, 
+                 "...wait3() returned %d; error: %s (%d) ",
+                 child_pid, STRERROR(errno), errno );
+    }
+        while  ( child_pid > 0 );
+
     signal ( SIGCHLD, VOIDSTAR reaper );
     return 0;
 }
@@ -711,11 +738,12 @@ cleanup ( SIGPARAM )
 
 
     TRACE  ( trace_file, POP_DEBUG, HERE, "cleanup signal handler called" );
+
     while  ( child_pid > 0 )
     {
         child_pid = wait3 ( &stts, WNOHANG, (struct rusage *) 0 );
         TRACE  ( trace_file, POP_DEBUG, HERE, 
-                 "wait3() returned %d; errno=%d",
+                 "...wait3() returned %d; errno=%d",
                  child_pid, errno );
     }
 
@@ -724,4 +752,22 @@ cleanup ( SIGPARAM )
 }
 
 
+int
+hupit ( SIGPARAM )
+{
+    TRACE  ( trace_file, POP_DEBUG, HERE, "hupit signal handler called" );
+
+    closelog();
+
+#ifdef SYSLOG42
+    openlog ( pname, 0 );
+#else
+    openlog ( pname, POP_LOGOPTS, LOG_DAEMON /*POP_FACILITY*/ );
+#endif
+
+    signal ( SIGHUP, VOIDSTAR hupit );
+    return 0;
+}
+
+
 #endif /* STANDALONE */
